<?php
/**
 * This file is a part of EasyWatermark Wordpress plugin.
 * @see readme.txt
 */
namespace szaleq\Plugins\EasyWatermark;

/**
 * Main plugin class 
 */
class Plugin extends Core
{
	/**
	 * @var string  plugin name
	 */
	protected static $pluginName = 'Easy Watermark';

	/**
	 * @var string  plugin slug used in setting names etc.
	 */
	protected static $pluginSlug = 'easy-watermark';

	/**
	 * @var string  plugin version
	 */
	protected static $version = '0.2';

	private $settings;

	private $messages = array();

	private $allowedMime = array('image/png', 'image/jpeg', 'image/gif');

	/**
 	 * Loads textdomain for translations,
 	 * adds wordpress actions and creates EasyWatermarkSettings object
	 *
	 * @return void
	 */
	public function __construct(){

		$this->add_action('admin_menu', 'add_media_page')
			->add_filter('media_row_actions', 'add_media_row_action', 10, 3)
			->add_filter('attachment_fields_to_edit', 'add_attachment_field', 10, 2)
			->add_action('add_attachment', 'add_watermark_after_upload');

		new EasyWatermarkSettings($this);
	}

	public function setSettings($settings){
		$this->settings = $settings;
	}

	/**
 	 * Adds wordpress media page
	 *
	 * @return void
	 */
	public function add_media_page(){
 		add_media_page( 'Easy Watermark', 'Easy Watermark', 'upload_files', 'easy-watermark', array($this, 'easy_watermark'));
	}

	/**
 	 * Creates 'Add watermark' link for each row in media library
	 *
	 * @return  array
	 */
	public function add_media_row_action($actions, $post, $detached){
		if($post->post_mime_type == 'image/jpeg' || $post->post_mime_type == 'image/png' || $post->post_mime_type == 'image/gif'){
			// Add link if it's supported image type
			$actions['add_watermark'] = '<a href="' . wp_nonce_url(admin_url('upload.php?page=easy-watermark&attachment_id='.$post->ID)) . '">'.__('Add watermark', 'easy-watermark').'</a>';
		}

		return $actions;
	}

	/**
 	 * Creates 'Add watermark' button in media edit view
	 *
	 * @return  array
	 */
	public function add_attachment_field($form_fields, $post){
		if($post->post_mime_type == 'image/jpeg' || $post->post_mime_type == 'image/png' || $post->post_mime_type == 'image/gif'){
			// Add link if it's supported image type
			$form_fields = array_reverse($form_fields);
			$form_fields['easy-watermark'] = array(
				'label' => '<a href="'.wp_nonce_url(admin_url('upload.php?page=easy-watermark&attachment_id='.$post->ID)).'" class="button-secondary">'.__('Add watermark', 'easy-watermark').'</a>',
				'input' => 'html',
				'html' => ' '
			);
		}

		return array_reverse($form_fields);
	}

	/**
 	 * Watermark image after it was uploaded.
	 * In fact this method only mark that there is an image to watermark,
	 * it's realy watermarked in wp_generate_attachment_metadata filter.
	 * See wp_generate_attachment_metadata() method below.
	 *
	 * @return  array
	 */
	public function add_watermark_after_upload($id){
		if($this->settings['general']['auto_add']){
			$this->watermark_uploaded = true;
			$this->uploaded_id = $id;
			$this->add_filter('wp_generate_attachment_metadata', null, 10, 2);
		}
	}

	/**
 	 * Filter used to add watermark
	 * add_attachment action is called before image resizing,
	 * so image was watermarked first, then resized.
	 * To avoid that, in add_attachment action we only mark,
	 * that image was uploaded and then it's watermarked in this filter
	 * which is called after resizing.
	 *
	 * @param   array
	 * @param   integer
	 * @return  array
	 */
	public function wp_generate_attachment_metadata($metadata, $id){
		if($this->watermark_uploaded && $this->uploaded_id == $id){
			$this->add_watermark($id);

			$this->watermark_uploaded = false;
			$this->uploaded_id = null;
		}

		return $metadata;
	}

	private function add_watermark($id){
		if($this->settings['image']['watermark_path'] != '' || $this->settings['text']['text'] == null){
			$post = get_post($id);
			$filepath = get_attached_file($id);

			if(!in_array($post->post_mime_type, $this->allowedMime)){
				$this->add_error('Invalid mime type.');
				return false;
			}

			return $this->create_watermark($filepath, $post->post_mime_type);
		}
		else {
			$this->add_error('No watermark image selected and no watermark text set.');
			return false;
		}
	}

	private function watermark_all(){
		global $wpdb;

		$images = $wpdb->get_results("
			SELECT ID, post_mime_type
			FROM $wpdb->posts
			WHERE post_type = 'attachment'
			AND post_mime_type LIKE 'image/%'
		");

		$output = '';
		if($images && ($this->settings['image']['watermark_path'] != '' || $this->settings['text']['text'])){
			foreach($images as $img){
				$filepath = get_attached_file($img->ID);
				if(!in_array($img->post_mime_type, $this->allowedMime)){
					$output .= sprintf(__('Not supported mime type of %s. Skipping...', 'easy-watermark'), '<strong>'.$filepath.'</strong>').'<br/>';
					continue;
				}

				if($this->create_watermark($filepath, $img->post_mime_type)){
					$output .= sprintf(__('Watermark successfully added to %s', 'easy-watermark'), '<strong>'.$filepath.'</strong>') . '<br/>';
				}
			}
		}
		else return false;

		$output = '<p>'.$output.'</p>';

		return $output;
	}

	private function create_watermark($imageFile, $imageType){
		$ew = $this->getEasyWatermark();
		$ew->setImagePath($imageFile)
			->setImageMime($imageType)
			->setOutputFile($imageFile)
			->setOutputMime($imageType);

		if(!$ew->create() || !$ew->saveOutput()){
			$this->add_error($ew->getError());
			return false;
		}

		return true;
	}

	private $ew;

	public function getEasyWatermark(){
		if(!($ew instanceof EasyWatermark)){
			$imageSettings = $this->settings['image'];
			$textSettings = $this->settings['text'];
			$ew = new EasyWatermark();

			$fontFile = dirname(__FILE__) . DIRECTORY_SEPARATOR . 'fonts' . DIRECTORY_SEPARATOR . $textSettings['font'];
			if(file_exists($fontFile))
				$textSettings['font'] = $fontFile;

			$wType = $this->settings['general']['watermark_type'];
			if($wType == 1 || $wType == 3)
				$ew->imageSet($imageSettings);
			if($wType == 2 || $wType == 3)
				$ew->textSet($textSettings);
		}

		return $ew;
	}

	public function easy_watermark(){
		?>
		<div class="wrap easy-watermark">
			<h2><?php _e('Easy Watermark', 'easy-watermark'); ?></h2>
		<?php
			if(isset($_GET['_wpnonce']) && wp_verify_nonce($_GET['_wpnonce'])){
				if(isset($_GET['attachment_id'])){
					if($this->add_watermark($_GET['attachment_id'])) :
					?>
			<div id="message" class="updated below-h2">
				<p><?php _e('Watermark successfully added.', 'easy-watermark'); ?> <a href="javascript:window.history.go(-1)"><?php _e('Go back', 'easy-watermark'); ?></a> <?php _e('or', 'easy-watermark'); ?> <a href="<?php echo get_edit_post_link($_GET['attachment_id']); ?>"><?php _e('go to edit page', 'easy-watermark'); ?></a></p>
			</div>
					<?php
					else :
						foreach($this->messages as $msg) :
					?>
			<div id="message" class="<?php echo $msg[0]; ?> below-h2">
				<p><?php _e($msg[1], 'easy-watermark'); ?></p>
			</div>
				 <a href="javascript:window.history.go(-1)"><?php _e('Go back', 'easy-watermark'); ?></a> or <a href="<?php echo get_edit_post_link($_GET['attachment_id']); ?>"><?php _e('go to edit page', 'easy-watermark'); ?></a>
					<?php
						endforeach;
					endif;
				}
				elseif(isset($_GET['watermark_all'])){
					if($output = $this->watermark_all()) :
					?>
			<div id="message" class="updated below-h2">
				<p><?php _e('Watermark successfully added.', 'easy-watermark'); ?> <a href="<?php echo admin_url('upload.php') ?>"><?php _e('Go to Media Library', 'easy-watermark'); ?></a></p>
			</div>
					<?
						echo $output;
					endif;
				}
			}
			else {
			?>
			<br/><a class="button-primary" href="<?php echo wp_nonce_url(admin_url('/upload.php?page=easy-watermark&watermark_all=1')); ?>"><?php _e('Add watermark to all images', 'easy-watermark'); ?></a><p class="description"><?php _e('Be carefull with that option. If some images alredy has watermark, it will be added though.'); ?></p>
			<?php
			}
		?>
		</div>
		<?php
	}

	public static function install($settings = null){
		if(empty($settings)) $settings = EasyWatermarkSettings::getDefaults();

		foreach($settings as $sectionName => $section){
			update_option(static::$pluginSlug.'-settings-'.$sectionName, $section);
		}
		update_option(static::$pluginSlug.'-version', static::$version);
	}

	public static function uninstall(){
		$settings = EasyWatermarkSettings::getDefaults();

		foreach($settings as $sectionName => $section){
			delete_option(static::$pluginSlug.'-settings-'.$sectionName);
		}
		delete_option(static::$pluginSlug.'-version');
	}

	public static function upgrade($version){
		$oldSettings = get_option(static::$pluginSlug.'-settings');
		$defaults = EasyWatermarkSettings::getDefaults();

		$imgTypes = array();
		foreach($oldSettings['image_types'] as $type){
			$imgTypes[] = $type;
		}

		$general = array(
			'auto_add' => $oldSettings['auto_add'],
			'image_types' => $imgTypes
		);

		switch($version){
			case '0.1.1':
				$image = array(
					'watermark_url' => $oldSettings['image']['url'],
					'watermark_id' => $oldSettings['image']['id'],
					'watermark_path' => $oldSettings['image']['path'],
					'watermark_mime' => $oldSettings['image']['mime'],
					'position_x' => $oldSettings['image']['position_x'],
					'position_y' => $oldSettings['image']['position_y'],
					'offset_x' => $oldSettings['image']['offset_x'],
					'offset_y' => $oldSettings['image']['offset_y'],
					'opacity' => $oldSettings['image']['opacity']
				);
				break;
			default:
				$image = array(
					'watermark_url' => $oldSettings['image']['url'],
					'watermark_id' => $oldSettings['image']['id'],
					'watermark_path' => $oldSettings['image']['path'],
					'watermark_mime' => $oldSettings['image']['mime'],
					'position_x' => $oldSettings['image']['position-horizontal'],
					'position_y' => $oldSettings['image']['position-vert'],
					'offset_x' => $oldSettings['image']['offset-horizontal'],
					'offset_y' => $oldSettings['image']['offset-vert'],
					'opacity' => $oldSettings['image']['alpha']
				);
				break;
		}

		$settings = array(
			'general' => array_merge($defaults['general'], $general),
			'image' => array_merge($defaults['image'], $image),
			'text' => $defaults['text']
		);

		delete_option(static::$pluginSlug.'-settings');
		static::install($settings);
	}

	private function add_error($msg){
		$this->messages[] = array('error', $msg);
	}

	private function add_info($msg){
		$this->messages[] = array('update', $msg);
	}
}
